"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceSequence = void 0;
const tslib_1 = require("tslib");
const context_1 = require("@loopback/context");
const rest_1 = require("@loopback/rest");
const lodash_1 = require("lodash");
const loopback4_authentication_1 = require("loopback4-authentication");
const loopback4_authorization_1 = require("loopback4-authorization");
const components_1 = require("./components");
const keys_1 = require("./keys");
const SequenceActions = rest_1.RestBindings.SequenceActions;
const isJsonString = (str) => {
    try {
        JSON.parse(str);
    }
    catch (e) {
        return false;
    }
    return true;
};
let ServiceSequence = /** @class */ (() => {
    let ServiceSequence = class ServiceSequence {
        constructor(
        // sonarignore:start
        findRoute, parseParams, invoke, send, reject, logger, authenticateRequest, checkAuthorisation, i18n) {
            this.findRoute = findRoute;
            this.parseParams = parseParams;
            this.invoke = invoke;
            this.send = send;
            this.reject = reject;
            this.logger = logger;
            this.authenticateRequest = authenticateRequest;
            this.checkAuthorisation = checkAuthorisation;
            this.i18n = i18n;
            /**
             * Optional invoker for registered middleware in a chain.
             * To be injected via SequenceActions.INVOKE_MIDDLEWARE.
             */
            this.invokeMiddleware = () => false;
        }
        async handle(context) {
            var _a;
            const requestTime = Date.now();
            try {
                const { request, response } = context;
                response.removeHeader('x-powered-by');
                this.logger.info(`Request ${request.method} ${request.url} started at ${requestTime.toString()}.
        Request Details
        Referer = ${request.headers.referer}
        User-Agent = ${request.headers['user-agent']}
        Remote Address = ${request.connection.remoteAddress}
        Remote Address (Proxy) = ${request.headers['x-forwarded-for']}`);
                const finished = await this.invokeMiddleware(context);
                if (finished)
                    return;
                const route = this.findRoute(request);
                const args = await this.parseParams(request, route);
                const authUser = await this.authenticateRequest(request);
                const isAccessAllowed = await this.checkAuthorisation(authUser === null || authUser === void 0 ? void 0 : authUser.permissions, request);
                if (!isAccessAllowed) {
                    throw new rest_1.HttpErrors.Forbidden("NotAllowedAccess" /* NotAllowedAccess */);
                }
                const result = await this.invoke(route, args);
                this.send(response, result);
            }
            catch (err) {
                this.logger.error(`Request ${context.request.method} ${context.request.url} errored out. Error :: ${JSON.stringify(err)} ${err}`);
                const error = this._rejectErrors(err);
                if (
                // sonarignore:start
                !(error.message &&
                    [
                        "Token Invalid" /* TokenInvalid */,
                        "Token Expired" /* TokenExpired */,
                        'TokenExpired',
                    ].includes(
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any
                    error.message.message))
                // sonarignore:end
                ) {
                    if (lodash_1.isString(error.message)) {
                        error.message = this.i18n.__({
                            phrase: error.message,
                            locale: (_a = process.env.LOCALE) !== null && _a !== void 0 ? _a : 'en',
                        });
                    }
                    else {
                        error.message =
                            error.message || 'Some error occured. Please try again';
                    }
                }
                this.reject(context, error);
            }
            finally {
                this.logger.info(`Request ${context.request.method} ${context.request.url} Completed in ${Date.now() - requestTime}ms`);
            }
        }
        // sonarignore:start
        /* eslint-disable @typescript-eslint/no-explicit-any */
        _rejectErrors(err) {
            // sonarignore:end
            if (!!err.table && !!err.detail) {
                if (err.code === '23505') {
                    // Postgres unique index error
                    return new rest_1.HttpErrors.Conflict(`Unique constraint violation error ! ${err.detail}`);
                }
                else if (err.code === '23503') {
                    // Postgres foreign key error
                    return new rest_1.HttpErrors.NotFound(`Related entity not found ! ${err.detail}`);
                }
                else if (err.code === '23502') {
                    // Postgres not null constraint error
                    return new rest_1.HttpErrors.NotFound(`Not null constraint violation error ! ${err.detail}`);
                }
                else {
                    return err;
                }
            }
            else if (err.message &&
                isJsonString(err.message) &&
                JSON.parse(err.message).error) {
                return JSON.parse(err.message).error;
            }
            else if (err.message &&
                err.message.message &&
                isJsonString(err.message.message) &&
                JSON.parse(err.message.message).error) {
                return JSON.parse(err.message.message).error;
            }
            else if (err.name && err.name === 'PubNubError') {
                return new rest_1.HttpErrors.UnprocessableEntity(`Pubnub returned with error ! ${JSON.stringify(err)}`);
            }
            else {
                return err;
            }
        }
    };
    tslib_1.__decorate([
        context_1.inject(SequenceActions.INVOKE_MIDDLEWARE, { optional: true }),
        tslib_1.__metadata("design:type", Function)
    ], ServiceSequence.prototype, "invokeMiddleware", void 0);
    ServiceSequence = tslib_1.__decorate([
        tslib_1.__param(0, context_1.inject(SequenceActions.FIND_ROUTE)),
        tslib_1.__param(1, context_1.inject(SequenceActions.PARSE_PARAMS)),
        tslib_1.__param(2, context_1.inject(SequenceActions.INVOKE_METHOD)),
        tslib_1.__param(3, context_1.inject(SequenceActions.SEND)),
        tslib_1.__param(4, context_1.inject(SequenceActions.REJECT)),
        tslib_1.__param(5, context_1.inject(components_1.LOGGER.LOGGER_INJECT)),
        tslib_1.__param(6, context_1.inject(loopback4_authentication_1.AuthenticationBindings.USER_AUTH_ACTION)),
        tslib_1.__param(7, context_1.inject(loopback4_authorization_1.AuthorizationBindings.AUTHORIZE_ACTION)),
        tslib_1.__param(8, context_1.inject(keys_1.SFCoreBindings.i18n)),
        tslib_1.__metadata("design:paramtypes", [Function, Function, Function, Function, Function, Object, Function, Function, Object])
    ], ServiceSequence);
    return ServiceSequence;
})();
exports.ServiceSequence = ServiceSequence;
//# sourceMappingURL=service-sequence.js.map